%
% z=arimasim(phi,theta,d,mu,sigmaa2,n)
%
% Randomly generate an n point time series according to an ARIMA(p,d,q) 
% process with given coefficients phi and theta, given mean mu, and
% given variance of the random noise.  
%
% In order to start the simulation process, we must pick some initial
% conditions.  This routine uses all 0's for the initial conditions, 
% but discards the first 20*(p+q) points in the time series so as to
% give the initial transients some time to die down.
% 
function z=arimasim(phi,theta,d,mu,sigmaa2,n)
%
% First, find p and q.
%
p=length(phi);
q=length(theta);
%
% Determine how long a sequence to generate, taking into account that
% we'll throw out the early part.
%
m=20*(p+q)+d+n;
%
% Compute the A_i's.
%
a=sqrt(sigmaa2)*randn(m,1);
%
% Initialize z to zeros.
%
w=zeros(m,1);
%
% Now, generate the sequence.
%
for j=max(p,q)+1:m,
  for i=1:p,
    w(j)=w(j)+phi(i)*w(j-i);
  end;
  w(j)=w(j)+a(j);
  for i=1:q,
    w(j)=w(j)-theta(i)*a(j-i);
  end;
end;
%
% Add the mean into w.
%
w=w+mu*ones(size(w));
%
% Now, integrate d times to get z.
%
z=w;
for i=1:d,
  z=invdiff(z);
end;
%
% Finally, take the last n points.
%
z=z(m-n+1:m);
