%
% [phi,theta,sigmaa2,muw,C]=fitarma(z,p,q,phi0,theta0)
%
% This routine fits an ARMA(p,q) model to the data z, using the method
% of least squares with a conditional least squares function.  
%
% Inputs:
%     z           Time series.
%     p           Number of autoregressive terms.
%     q           Number of moving average terms.
%     phi0        Initial guess for phi.
%     theta0      Initial guess for theta.
%
% Outputs:
%     phi         Optimal phi.
%     theta       Optimal theta.
%     sigmaa2     Estimated variance of the white noise.
%     muw         Mean of z (removed before fitting the ARMA process.)
%     C           Covariance matrix for the estimated parameters.
%
function [phi,theta,sigmaa2,muw,C]=fitarma(z,p,q,phi0,theta0)
%
% Globals.
%
global P;
global Q;
global W;
%
% Remove the mean from the data.
%
muw=mean(z);
W=z-muw*ones(size(z));
%
% Solve the least squares problem.
%
P=p;
Q=q;
beta0=[phi0; theta0];
betastar=lm('finda','findaj',beta0,1.0e-8,100);
%
% Strip out phi and theta.
%
phi=betastar(1:p);
theta=betastar(p+1:p+q);
%
% Get the estimate of sigmaa2.
%
sigmaa2=norm(finda(betastar))^2/(length(W)-q);
%
% Get C.
%
J=findaj(betastar);
C=2*sigmaa2*inv(J'*J);


