%
% xstar=lm(func,jac,x0,tol,maxiter)
%
% Use the Levenberg-Marquardt algorithm to minimize 
%
%  f(x)=sum(F_i(x)^2)
%
%    func         name of the function F(x)
%    jac          name of the Jacobian function J(x)
%    x0           initial guess
%    tol          stopping tolerance
%    maxiter      maximum number of iterations allowed
%
function pstar=lm(func,jac,p0,tol,maxiter)
%
%  Initialize p and oldp.  
%
p=p0;
fp=norm(feval(func,p),2)^2;
oldp=p0*2;
oldfp=fp*2;
%
% Initialize lambda.
%
lambda=0.0001;
%
% The main loop.  While the current solution isn't good enough, keep
% trying...  Stop after maxiter iterations in the worst case.
%
  iter=0;
  while (iter <= maxiter)
%
% To see the estimated parameters at the start of each iteration, uncomment
% the following line:
%
%     p
%
% Compute the Jacobian.
%
    J=feval(jac,p);
%
% Compute G=(J'*J+lambda*I)
%
    G=J'*J+lambda*eye(length(p));
%
% Compute rhs=-J'*f
%
    rhs=-J'*feval(func,p);
%
% Check the termination criteria.
%
    if ((norm(rhs,2)< sqrt(tol)*(1+abs(fp))) & ...
        (abs(oldfp-fp)<tol*(1+abs(fp))) & ...
        (norm(oldp-p,2)<sqrt(tol)*(1+norm(p,2)))),
      pstar=p;
      return;
    end;
%
% Compute the step s.
%
    s=G\rhs;
%
% See whether this improves chisq or not.
%
    fpnew=norm(feval(func,p+s),2)^2;
%
% If this improves f, then make the step, and decrease lambda and make
% the step.
%
    if (fpnew < fp)
      oldp=p;
      oldfp=fp;
      p=p+s;
      fp=fpnew;
      lambda=lambda/2;
      if (lambda <10^(-8))
        lambda=1.0e-8;
      end;
      iter=iter+1;
    else
%
% Didn't improve f, increase lambda, and try again.
%
      lambda=lambda*2;
      if (lambda >10^6)
        lambda=10^6
      end;
      iter=iter+1;
    end;
%
%  end of the loop.
%
  end;
%
%  Return, max iters exceeded.
%
disp('max iters exceeded');
pstar=p;




